#include <string>
#include <fstream>
#include <iostream>
#include <pthread.h>

#ifdef USE_TAO
#include "ServerC.h"
#include "GlyphC.h"
#endif

#ifdef USE_OMNI
#include "Server.hh"
#include "Glyph.hh"
#endif

#ifdef USE_ORBACUS
#include <OB/CORBA.h>
#include "Server.h"
#include "Glyph.h"
#endif

#define SCENE_DEPTH 100
#define LOOP_COUNT 1000
using namespace Fresco;

int
main (int argc, char *argv[])
{
  CORBA::ORB_var orb = CORBA::ORB_init (argc, argv);

  ifstream in("iorfile");
  string ior;
  in >> ior;

  Server_var srv = Server::_narrow (orb->string_to_object (ior.c_str ()));

  if (CORBA::is_nil (srv.in()))
    {
      cout << "Can't get to the server" << endl;
      return -1;
    }

  GlyphFactory_var factory = srv->get_factory ();
  if (CORBA::is_nil (factory.in()))
    {
      cout << "Can't get glyph factory" << endl;
      return -1;
    }
  
  Screen_var screen = srv->get_screen ();
  if (CORBA::is_nil (screen.in()))
    {
      cout << "Can't get the screen" << endl;
      return -1;
    }

  Traversal_var traversal = factory->make_traversal ();

  // Create a list of glyphs.  
  Glyph_var head, g;

  head = factory->make_glyph ();
  g = head;
  for (CORBA::Long i = 0; i < SCENE_DEPTH; i++)
    {
      Glyph_var c;
      c = factory->make_glyph ();
      g->body (c.in());
      g = c;
    }

  cout << "Time :" << screen->traverse (LOOP_COUNT / 100, traversal.in(), head.in()) << endl;
  cout << "Time :" << screen->traverse (LOOP_COUNT / 50, traversal.in(), head.in()) << endl;
  cout << "Time :" << screen->traverse (LOOP_COUNT / 10, traversal.in(), head.in()) << endl;
  cout << "Time :" << screen->traverse (LOOP_COUNT, traversal.in(), head.in()) << endl;
  return 0;
}
